function [rhs] = assembleRhs(f, H, opt)

if nargin < 2
    opt = struct();
end
rhs = struct;
rhs.f = f;
rhs.geometry = H.geometry.tensor;

% right hand side as tensor product
[rhs] = interpolateF(H, rhs, opt);

% SVD of weight function (without low rank)
[rhs]  = lowRankSVDf(rhs, opt);
% univariate quadrature

if rhs.geometry.dim == 2
    rhs = univariateIntegralSf(rhs);
    rhs = univariateIntegralTf(rhs);
    rhs.interp.X = rhs.interp.X(logical(H.sInner),:);
    rhs.interp.Y = rhs.interp.Y(logical(H.tInner), :);
elseif rhs.geometry.dim == 3
    rhs = univariateIntegralXf(rhs);
    rhs = univariateIntegralYf(rhs);
    rhs = univariateIntegralZf(rhs);
end

rhs.X = rhs.X(logical(H.boundary.xInner),:);
rhs.Y = rhs.Y(logical(H.boundary.yInner),:);
rhs.Z = rhs.Z(logical(H.boundary.zInner),:);
rhs.b = zeros(sum(H.boundary.xInner)*sum(H.boundary.yInner)*sum(H.boundary.zInner),1);
for i = 1:rhs.R(1)
    for j = 1:rhs.R(2)
        for k = 1:rhs.R(3)
            rhs.b = rhs.b + kron(rhs.Z(:,k), kron(rhs.Y(:,j), rhs.X(:,i)))*rhs.SVDWeights(i,j,k);
        end
    end
end

% assemble kroenecker product
% rhs.vec = zeros(size(rhs.interp.X,1)*size(rhs.interp.Y,1),1);
% for r = 1:rhs.interp.R
%     rhs.vec = rhs.vec + kron(rhs.interp.X(:,r), rhs.interp.Y(:,r));
% end
% % f_rhs = f_rhs(logical(kron(real(H.sInner),real(H.tInner))));
% % f_rhs = cell(2, R);
% for r = 1:R
%    f_rhs{1,r} = fY(:,r);
%    f_rhs{2,r} = fX(:,r);
% end
end