function [knots2, degree2] = enlargenNURBSSpace(knots, degree, S2, D2)

    
    
    uniqueKnots = unique(knots);
    multiplicities = histc(knots, uniqueKnots);
        
    
    %% for det Jac(G)
    % Continuity class of original spline in a knot:
    %   C^k, where k = degree - multiplicity of the knot
    % Continuity class of f1 * d/dt f2 in a knot:
    %   C^k, where k = degree - multiplicity - 1
    % Continuity of new spline in a knot:
    %   C^k, where k = degree2 - multiplicity2 
    % => multiplicity2 = degree2 - degree + multiplicity + 1
    %   with degree2 = 2*degree - 1 
    % => multiplicity2 = degree + multiplicity
    
    % degree2 = 2*degree - 1;
    % multiplicities2 = multiplicity + degree
    
    %% Angelos paper for stiffness matrix:
    % Continuity class of original spline in a knot:
    %   C^k, where k = degree - multiplicity of knot
    % Continuity class of abs(det(Jac(G)))*inv(Jac(G))*inv(Jac(G))'
    %   C^k, where k = max(1, 1 + multiplicity in original geometry +
    %   degree - degree2)
    
    if D2 == 0
        degree2 = 2*degree+1;
    else
        degree2 = D2;
    end 
     multiplicities2 = max(1, degree2 - degree + multiplicities +1);
    % multiplicities2 = (multiplicities + degree2) - multiplicities;
   
    % Multiplicity of first and last knot must be degree2+1
    multiplicities2(1) = degree2+1;
    multiplicities2(end) = degree2+1;

    %multiplicity is only allowed up to degree
    a = find(multiplicities2(2:end-1)>degree2);
    multiplicities2(1+a) = degree2;
    multiplicitiesNew = multiplicities2;
     
    % enlargen spline space with new knots
    for k = 1:S2
        addKnots = (uniqueKnots(2:end)-uniqueKnots(1:end-1))/2 + uniqueKnots(1:end-1);
        multiplicitiesNew2 = ones(1,numel(multiplicitiesNew)+numel(addKnots));
        multiplicitiesNew2(1:2:end) = multiplicitiesNew;
        uniqueKnots = sort([uniqueKnots,addKnots]);
        multiplicitiesNew = multiplicitiesNew2;
    end

    
    % Generate right knot vector
    knots2 = zeros(1, sum(multiplicitiesNew));
    k = 0;
    for i = 1:numel(uniqueKnots)
        knots2(k + (1:multiplicitiesNew(i))) = uniqueKnots(i);
        k = k + multiplicitiesNew(i);
    end
    
%     dim = numel(knots) - degree - 1;
%     dim2 = sum(multiplicities2) - degree2 - 1;
%     
%     %% Test for one dimensional Bspline interpolation
%     
%     % Generate Greville points
%     samplePoints = zeros(1, dim2);
%     for i = 1:dim2
%         samplePoints(i) = sum(knots2(i+1:i+degree2)) / (degree2);
%     end
%     
%     basis2SampleValues = evalBSpline(knots2, degree2, samplePoints);
%     
%     weights1 = rand(2, dim);
%     
%     rhsSampleValues = (weights1(1,:) * evalBSpline(knots, degree, samplePoints)) ...
%         .* (weights1(2,:) * evalBSplineDeriv(knots, degree, samplePoints));
%     
%     weights2 = (basis2SampleValues' \ rhsSampleValues')';
%     
%     ttPlot = linspace(knots(1), knots(end), 1001);
%     yyPlot1 = (weights1(1,:) * evalBSpline(knots, degree, ttPlot)) .* (weights1(2, :) * evalBSplineDeriv(knots, degree, ttPlot));
%     yyPlot2 = weights2 * evalBSpline(knots2, degree2, ttPlot);
% 
%     figure;
%     plot(ttPlot, yyPlot1, 'b', ttPlot, yyPlot2, 'r--', samplePoints, zeros(size(samplePoints)), 'kx', 'MarkerSize', 10);
%     legend('f1*df2/dt in smaller space', 'Approximation in larger space', 'Greville points');

end

