
function splines = evalBSpline(knots, degree, eta)
% EVALBSPLINE Evaluate all B-Spline basis functions in a number of points.
%
% The number of splines is numel(knots)-degree-1.
% splines(i,j) = value of i-th basis function, evaluated in eta(j).


    % Number of basis functions
    m = numel(knots) - degree - 1;
    
    splines = zeros(m, length(eta));
    
    % recursive calculation of B-Spline values
    if degree == 0
        for i = 1:m
            splines(i, :) = (knots(i) <= eta) & (eta < knots(i+1));
        end
        if knots(1) < knots(end)
            % constant spline on last nonempty interval must be
            % left-continuous in its endpoint
            i = find(knots < knots(end), 1, 'last');
            splines(i, eta == knots(end)) = 1;
        end
    else
        prevSplines = evalBSpline(knots, degree-1, eta);
        for i = 1:m
            if knots(i+degree) > knots(i)
                splines(i, :) = (eta - knots(i)) / (knots(i+degree) - knots(i)) .* prevSplines(i, :);
%             else
%                 splines(i, :) = (knots(i+degree) == eta) .* prevSplines(i, :);
            end
            if knots(i+degree+1) > knots(i+1)
                splines(i, :) = splines(i, :) + (knots(i+degree+1) - eta) / (knots(i+degree+1) - knots(i+1)) .* prevSplines(i+1, :);
%             else
%                 splines(i,:) = splines(i,:) + (knots(i+1) == eta) .* prevSplines(i+1, :);
            end
        end
        splines(m, eta == knots(end)) = 1;
        %splines = sparse(splines);
    end
    