function [G] = fitNURBS(G,fig)
%% fits two dimensional NURBS from geometry structure G into tensor product structure, such that weights \approx \otimes_i=1^d weights_i
% input: geometry structure G as assembled in readGeo, fig=1 if domain
% should be plotted, 0 else
% return: geometry struct G with new substructure .tensor with fields
%   dim:        same dimension
%   knots:      same knots
%   degree:     same degree
%   n:          same number of splines
%   Tweights:   rounded tt_tensor of the weight matrix/tensor, such that
%               weights is approximated by kroenecker product of d 1D arrays
%   weight:     reassembled weight tensor, approximation to and same size as
%               G.weight
%   controlPoints: same control points as before or adapted control points,
%               if wished for (!!! not yet available in 3D)



% if isfield(G, 'tensor')
%     warning('Geometry already possesses tensor structure.')
%     m = input('overwrite tensors? [y]/[n]\n','s');
%     if m == 'n'
%         return;
%     end
% end
G.tensor = struct;
G.tensor.dim = G.dim;
G.tensor.knots = G.knots;
G.tensor.degree = G.degree;
G.tensor.n = G.n;

if G.dim == 2
    G.tensor.Tweights = round(tt_tensor(G.weight), 1e-5,1);
    G.tensor.weight = reshape(kron(G.tensor.Tweights{2}, G.tensor.Tweights{1}), G.n);
    gg1 = generateGrevillePoints(G.knots{1}, G.degree(1));
    gg2 = generateGrevillePoints(G.knots{2}, G.degree(2));
    x = evalBSpline(G.knots{1}, G.degree(1), gg1);
    y = evalBSpline(G.knots{2}, G.degree(2), gg2);
    xT = evalNURBS(G.knots{1}, G.degree(1), G.tensor.Tweights{1}', gg1);
    yT = evalNURBS(G.knots{2}, G.degree(2), G.tensor.Tweights{2}', gg2);


    X = reshape(x'*(G.controlPoints(:,:,1).*G.weight)*y./(x'*G.weight*y),prod(G.n),1);
    Y = reshape(x'*(G.controlPoints(:,:,2).*G.weight)*y./(x'*G.weight*y),prod(G.n),1);
    X2 = kron(yT',xT');

    G.tensor.controlPoints(:,:,1) = reshape(X2\X, G.n);
    G.tensor.controlPoints(:,:,2) = reshape(X2\Y, G.n);
elseif G.dim == 3
    weightR = reshape(G.weight, G.n(1), G.n(2), G.n(3));
    G.tensor.Tweights = round(tt_tensor(weightR), 1e-5, 1);
    G.tensor.weight = kron(kron(G.tensor.Tweights{3},G.tensor.Tweights{2}),G.tensor.Tweights{1})';
%     m = input('Calculate new control points? [y]/[n] \n', 's');
%     if m == 'y'
%         gg1 = generateGrevillePoints(G.knots{1}, G.degree(1));
%         gg2 = generateGrevillePoints(G.knots{2}, G.degree(2));
%         gg3 = generateGrevillePoints(G.knots{3}, G.degree(3));
%         ss = evalBSpline(G.knots{1}, G.degree(1), gg1);
%         tt = evalBSpline(G.knots{2}, G.degree(2), gg2);
%         ll = evalBSpline(G.knots{3}, G.degree(3), gg3);
%         X = ((kron(ll,kron(tt,ss)).*reshape(G.weight, prod(G.n),1))'./(kron(ll,kron(tt,ss))'*reshape(G.weight,prod(G.n),1)))*reshape(G.controlPoints(:,:,:,1),prod(G.n),1);
%         Y = ((kron(ll,kron(tt,ss)).*reshape(G.weight, prod(G.n),1))'./(kron(ll,kron(tt,ss))'*reshape(G.weight,prod(G.n),1)))*reshape(G.controlPoints(:,:,:,2),prod(G.n),1);
%         Z =  ((kron(ll,kron(tt,ss)).*reshape(G.weight, prod(G.n),1))'./(kron(ll,kron(tt,ss))'*reshape(G.weight,prod(G.n),1)))*reshape(G.controlPoints(:,:,:,3),prod(G.n),1);
%         X2 = ((kron(ll,kron(tt,ss)).*reshape(G.weight, prod(G.n),1))'./(kron(ll,kron(tt,ss))'*reshape(G.weight,prod(G.n),1)));
%         G.tensor.controlPoints = [X2\X, X2\Y, X2\Z];
%     else
        G.tensor.controlPoints = reshape(G.controlPoints, prod(G.n),3);
%     end
else
    error('No valid dimension given');
end


if nargin < 2
    return;
elseif fig == 1
    if G.dim == 2
        p = plotNDomain(G, 'blue');
        plotNDomain(G.tensor, ['red', '--'], p);
    else
        p = plot3(reshape(G.controlPoints(:,:,:,1),prod(G.n),1), reshape(G.controlPoints(:,:,:,2),prod(G.n),1), reshape(G.controlPoints(:,:,:,3),prod(G.n),1),['red','o']);
        hold on;
        plot3(G.tensor.controlPoints(:,1), G.tensor.controlPoints(:,2), G.tensor.controlPoints(:,3),['black','*']);
        p = plotNDomain3(G.tensor,p);
        p = plotNLines3(G, 'red',p);
        plotNLines3(G.tensor, ['black', '--'], p);
    end
end

end