function [rhs] = interpolateF(H, rhs, opt)
%UNTITLED Summary of this function goes here
%   Detailed explanation goes here
if ~isfield(opt, 'splinespace2') || isempty(opt.splinespace2)
    opt.splinespace2 = 0;
end

    rhs.weightFun.knots = cell(H.geometry.dim);
    rhs.weightFun.degree = zeros(H.geometry.dim,1);
    rhs.weightFun.weight = cell(H.geometry.dim,1);
    rhs.weightFun.n = zeros(H.geometry.dim,1);
    for i = 1:H.geometry.dim
        [knots2, degree2] = enlargenNURBSSpace(H.geometry.knots{i}, H.geometry.degree(i), opt.splinespace2);
        rhs.weightFun.knots{i} = knots2;
        rhs.weightFun.degree(i) = degree2;
        rhs.weightFun.n(i) = numel(knots2)-degree2-1;
        rhs.weightFun.weight{i} = ones(1, rhs.weightFun.n(i));
    end
  
    grevillePoints = cell(H.geometry.dim,1);
    grevilleValues = cell(H.geometry.dim,1);
    grevilleDerivs = cell(H.geometry.dim,1);
    grevilleValues2 = cell(H.geometry.dim,1);
    
   %% Generate Greville points and evaluate basis splines there
    
   for i=1:H.geometry.dim
       grevillePoints{i} = generateGrevillePoints(rhs.weightFun.knots{i}, rhs.weightFun.degree(i));
     % Compute basis spline values in greville points
    grevilleValues{i} =  sparse(evalNURBS      (rhs.geometry.knots{i}, rhs.geometry.degree(i), rhs.geometry.Tweights{i}',  grevillePoints{i}));
    grevilleDerivs{i} =  sparse(evalNURBSDeriv (rhs.geometry.knots{i}, rhs.geometry.degree(i), rhs.geometry.Tweights{i}',  grevillePoints{i}));
    grevilleValues2{i} = sparse(evalNURBS      (rhs.weightFun.knots{i}, rhs.weightFun.degree(i), rhs.weightFun.weight{i}', grevillePoints{i}));
   end
    %% Setup equation matrix and right hand sides
    
    if H.geometry.dim == 2
        % Each row in grevillePoints2D contains [s t] coordinates of a
        % combination of a point in grevillePointsS and one in grevillePointsT.
        % The number of rows is dimS2*dimT2.

        % Allocate f
        eqMatrix = kron(grevilleValues2{2}', grevilleValues2{1}');
        jac1 = grevilleDerivs{1}' * rhs.geometry.controlPoints(:,:,1) * grevilleValues{2};
        jac2 = grevilleDerivs{1}'* rhs.geometry.controlPoints(:,:,2) * grevilleValues{2};
        jac3 = grevilleValues{1}' * rhs.geometry.controlPoints(:,:,1) * grevilleDerivs{2}; %21
        jac4 = grevilleValues{1}' * rhs.geometry.controlPoints(:,:,2) * grevilleDerivs{2};

        absDet = abs(jac1.*jac4 - jac2.*jac3);
        f_temp = rhs.f(grevilleValues{1}'*rhs.geometry.controlPoints(:,:,1)*grevilleValues{2}, grevilleValues{1}'*rhs.geometry.controlPoints(:,:,2)*grevilleValues{2});

        eqRhs = reshape(absDet.*f_temp,prod(rhs.weightFun.n),1);
%         eqFF = reshape(f_temp,prod(rhs.geometry.n),1);
%         FFweight = reshape(eqMatrix\eqFF, dimS2, dimT2);

        %% Compute weight matrix
        vecWeights = eqMatrix \ eqRhs;
        rhs.weightMat = reshape(vecWeights, rhs.weightFun.n(1), rhs.weightFun.n(2));
    elseif H.geometry.dim == 3
%         eqMatrix = kron(grevilleValues2{3},kron(grevilleValues2{2},grevilleValues2{1}))';
        jac11 = kron(grevilleValues{3}, kron(grevilleValues{2}, grevilleDerivs{1}))'*rhs.geometry.controlPoints(:,1);
        jac12 = kron(grevilleValues{3}, kron(grevilleValues{2}, grevilleDerivs{1}))'*rhs.geometry.controlPoints(:,2);
        jac13 = kron(grevilleValues{3}, kron(grevilleValues{2}, grevilleDerivs{1}))'*rhs.geometry.controlPoints(:,3);
        jac21 = kron(grevilleValues{3}, kron(grevilleDerivs{2}, grevilleValues{1}))'*rhs.geometry.controlPoints(:,1);
        jac22 = kron(grevilleValues{3}, kron(grevilleDerivs{2}, grevilleValues{1}))'*rhs.geometry.controlPoints(:,2);
        jac23 = kron(grevilleValues{3}, kron(grevilleDerivs{2}, grevilleValues{1}))'*rhs.geometry.controlPoints(:,3);
        jac31 = kron(grevilleDerivs{3}, kron(grevilleValues{2}, grevilleValues{1}))'*rhs.geometry.controlPoints(:,1);
        jac32 = kron(grevilleDerivs{3}, kron(grevilleValues{2}, grevilleValues{1}))'*rhs.geometry.controlPoints(:,2);
        jac33 = kron(grevilleDerivs{3}, kron(grevilleValues{2}, grevilleValues{1}))'*rhs.geometry.controlPoints(:,3);
        absDet = abs(jac11.*jac22.*jac33 + jac12.*jac23.*jac31 + jac21.*jac32.*jac13 - jac13.*jac22.*jac31 - jac12.*jac21.*jac33 - jac11.*jac23.*jac32);
        valTemp = kron(grevilleValues{3}, kron(grevilleValues{2}, grevilleValues{1}))'*rhs.geometry.controlPoints;
        eqRhs = rhs.f(valTemp(:,1), valTemp(:,2), valTemp(:,3)).*absDet;
        MM = {grevilleValues2{1}';grevilleValues2{2}';grevilleValues2{3}'};
        tt_rhs = tt_tensor(reshape(eqRhs, [size(grevilleValues2{1}',1),size(grevilleValues2{2}',1),size(grevilleValues2{3}',1)]),1e-16);
        sol_rhs = amen_block_solve({MM},{tt_rhs},1e-9, 'kickrank', 2, 'resid_damp', 1e1, 'nswp', 4, 'exitdir', -1);        
%       vecWeights = eqMatrix \eqRhs;
        rhs.weightMat = full(sol_rhs,H.weightFun.n');
    end
    %% Plot error
     if opt.plotW == 1
         if H.geometry.dim == 2
             ss = linspace(rhs.geometry.knots{1}(1), rhs.geometry.knots{1}(end), 101);
             tt = linspace(rhs.geometry.knots{2}(1), rhs.geometry.knots{2}(end), 101);

             [SS, TT] = meshgrid(ss, tt);

             XX = zeros(size(SS));
             YY = zeros(size(SS));
             ZZ = zeros(size(SS));
             FF = zeros(size(SS));
             FFWeights = zeros(size(SS));
             ZZWeights = zeros(size(SS));
             for i = 1:size(SS,1)
                 for j = 1:size(SS,2)

                    valS  = evalNURBS     (rhs.geometry.knots{1}, rhs.geometry.degree(1), rhs.geometry.Tweights{1}', SS(i,j));
                    derS  = evalNURBSDeriv(rhs.geometry.knots{1}, rhs.geometry.degree(1), rhs.geometry.Tweights{1}', SS(i,j));
                    valS2 = evalNURBS     (rhs.weightFun.knots{1}, rhs.weightFun.degree(1), rhs.weightFun.weight{1}', SS(i,j));
                    valT  = evalNURBS     (rhs.geometry.knots{2}, rhs.geometry.degree(2), rhs.geometry.Tweights{2}', TT(i,j));
                    derT  = evalNURBSDeriv(rhs.geometry.knots{2}, rhs.geometry.degree(2), rhs.geometry.Tweights{2}', TT(i,j));
                    valT2 = evalNURBS     (rhs.weightFun.knots{2}, rhs.weightFun.degree(2), rhs.weightFun.weight{2}', TT(i,j));

                    XX(i,j) = valS' * rhs.geometry.controlPoints(:,:,1) * valT;
                    YY(i,j) = valS' * rhs.geometry.controlPoints(:,:,2) * valT;

                    jacobi(1, 1) = derS' * rhs.geometry.controlPoints(:,:,1) * valT;
                    jacobi(2, 1) = derS' * rhs.geometry.controlPoints(:,:,2) * valT;
                    jacobi(1, 2) = valS' * rhs.geometry.controlPoints(:,:,1) * derT;
                    jacobi(2, 2) = valS' * rhs.geometry.controlPoints(:,:,2) * derT;

                    fval = rhs.f(XX(i,j),YY(i,j));
                    FF(i,j) = fval;
                    absDet = abs(det(jacobi));         

                    ZZ(i,j) = fval*absDet;
                    ZZWeights(i,j) = valS2'*rhs.interp.weightMat*valT2;
                    FFWeights(i,j) = valS2'*FFweight*valT2;

                 end
             end
             figure;
             surf(XX, YY, log(abs(ZZ-ZZWeights)))
            title('Error for Rhs Weight Matrix Interpolation')
            zlabel('Log Scale of Absolute Error')
         elseif H.geometry.dim == 3
            h = 50;
            xx = cell(H.geometry.dim,1);
            values = cell(H.geometry.dim,1);
            derivs = cell(H.geometry.dim,1);
            values2 = cell(H.geometry.dim,1);
            for i = 1:3
                xx{i} = linspace(rhs.geometry.knots{i}(1), rhs.geometry.knots{i}(end), h);
                values{i} = sparse(evalNURBS(rhs.geometry.knots{i}, rhs.geometry.degree(i), rhs.geometry.Tweights{i}', xx{i}));
                derivs{i} = sparse(evalNURBSDeriv(rhs.geometry.knots{i}, rhs.geometry.degree(i), rhs.geometry.Tweights{i}', xx{i}));
                values2{i} = sparse(evalNURBS(rhs.weightFun.knots{i}, rhs.weightFun.degree(i), rhs.weightFun.weight{i}',xx{i}));
            end
            jac11 = kron(values{3}, kron(values{2}, derivs{1}))'*rhs.geometry.controlPoints(:,1);
            jac12 = kron(values{3}, kron(derivs{2}, values{1}))'*rhs.geometry.controlPoints(:,1);
            jac13 = kron(derivs{3}, kron(values{2}, values{1}))'*rhs.geometry.controlPoints(:,1);
            jac21 = kron(values{3}, kron(values{2}, derivs{1}))'*rhs.geometry.controlPoints(:,2);
            jac22 = kron(values{3}, kron(derivs{2}, values{1}))'*rhs.geometry.controlPoints(:,2);
            jac23 = kron(derivs{3}, kron(values{2}, values{1}))'*rhs.geometry.controlPoints(:,2);
            jac31 = kron(values{3}, kron(values{2}, derivs{1}))'*rhs.geometry.controlPoints(:,3);
            jac32 = kron(values{3}, kron(derivs{2}, values{1}))'*rhs.geometry.controlPoints(:,3);
            jac33 = kron(derivs{3}, kron(values{2}, values{1}))'*rhs.geometry.controlPoints(:,3);
            valTemp = kron(values{3}, kron(values{2}, values{1}))'*rhs.geometry.controlPoints;
            w = abs(jac11.*jac22.*jac33 + jac12.*jac23.*jac31 + jac21.*jac32.*jac13 - jac13.*jac22.*jac31 - jac12.*jac21.*jac33 - jac11.*jac23.*jac32).*rhs.f(valTemp(:,1), valTemp(:,2), valTemp(:,3)); 
            absWeights = kron(values2{3}, kron(values2{2}, values2{1}))'*reshape(rhs.weightMat, prod(rhs.weightFun.n),1);
            
            figure
            plot(w,'*');
            hold on
            plot(absWeights, 'ro');
            
         end
     end

end

