function [rhs]  = lowRankSVDf(rhs, opt)

if nargin < 2
    opt = struct();
end
if ~isfield(opt, 'lowRank') || isempty(opt.lowRank)
    opt.lowRank = 0;
end
if ~isfield(opt, 'rankTol') || isempty(opt.rankTol)
    if opt.lowRank == 1
        opt.rankTol = 1e-5;
        fprintf('Set truncation tolerance for low rank approximation to %d.\n', opt.rankTol);
    else
        opt.rankTol = inf;
    end
end
if ~isfield(opt, 'discardFull') || isempty(opt.discardFull)
    opt.discardFull = 0;
end
if ~isfield(opt, 'sizeLowRank') || isempty(opt.sizeLowRank)
    opt.sizeLowRank = 15;
end

rhs.Rmax = min(rhs.geometry.n);
% assemble matrices for the decomposition
if rhs.geometry.dim == 2
    [U, weights1D, V] = svd(rhs.weightMat);
    weights1D = diag(weights1D);
    if opt.lowRank == 1
        rhs.R = find(weights1D < opt.rankTol,1)-1;
    else
        rhs.R = rhs.Rmax;
    end
    rhs.SVDU = U(:,1:rhs.R);
    rhs.SVDV = V(:,1:rhs.R);
    rhs.SVDWeights = sqrt(weights1D(1:rhs.R));
    if opt.discardFull == 1
        rhs = rmfield(rhs, 'weightMat');
    end
elseif rhs.geometry.dim == 3
    [rhs.SVDU, rhs.SVDWeights] = mlsvd(rhs.weightMat);
    rhs.R(1) = size(rhs.SVDWeights,1);
    rhs.R(2) = size(rhs.SVDWeights,2);
    rhs.R(3) = size(rhs.SVDWeights,3);
    if opt.discardFull == 1
        rhs = rmfield(rhs, 'weightMat');
    end
end
end

