 function [H, opt] = matrixFormat(H, opt)
%% either put together whole stiffness matrix or assemble ingredients in 
%% the right format for TT Toolbox
if nargin < 2
    opt = struct();
end


if H.geometry.dim == 2
    if ~isfield(opt, 'TT') || isempty(opt.TT)
    opt.TT = 0;
    end
    if opt.stiffness == 1
        % size for each integral matrix (second and third use the same low rank
        % weights)
        H.stiffness.R = [H.stiffness.R(1),H.stiffness.R(2),H.stiffness.R(2),H.stiffness.R(3)];

        % zero boundary condition
         H.stiffness.X = H.stiffness.X(logical(H.sInner), logical(H.sInner),:,:);
         H.stiffness.Y = H.stiffness.Y(logical(H.tInner), logical(H.tInner),:,:);

        if opt.TT == 1
            % assemble cell structure for TT Toolbox
            H.stiffnessMat = cell(2, sum(H.stiffness.R));
            k = 1;
            for i = 1:4
                for r = 1:H.stiffness.R(i)
                    H.stiffnessMat{1,k} = sparse(H.stiffness.Y(:, :, r, i));
                    H.stiffnessMat{2,k} = sparse(H.stiffness.X(:, :, r, i));
                    k = k+1;
                end
            end
        elseif isfield(opt,'discardFull') && opt.discardFull == 0
            % kroenecker product for complete stiffness matrix
            H.stiffnessMat = sparse(size(H.stiffness.X,1)*size(H.stiffness.Y,1),size(H.stiffness.X,1)*size(H.stiffness.Y,1));
            for i = 1:4
                for r = 1:H.stiffness.R(i)
                    H.stiffnessMat = sparse(H.stiffnessMat + kron(H.stiffness.Y(:,:,r,i), H.stiffness.X(:,:,r,i)));
                end
            end
        end
    end
    if opt.mass == 1

        H.mass.X = H.mass.X(logical(H.sInner), logical(H.sInner),:);
        H.mass.Y = H.mass.Y(logical(H.tInner), logical(H.tInner),:);
        if opt.TT == 1
            H.massMat = cell(2,H.mass.R);
            for r = 1:H.mass.R
                H.massMat{1, r} = sparse(H.mass.Y(:,:,r));
                H.massMat{2, r} = sparse(H.mass.X(:,:,r));
            end
        else
            H.massMat = sparse(size(H.mass.X,1)*size(H.mass.Y,1),size(H.mass.X,1)*size(H.mass.Y,1));
            for r = 1:H.mass.R
                H.massMat = sparse(H.massMat + kron(H.mass.X(:,:,r), H.mass.Y(:,:,r)));
            end
        end
    end

elseif H.geometry.dim == 3  
    if isfield(opt, 'discardFull') && opt.discardFull == 0
        if opt.mass == 1 
        H.massMat = sparse(sum(H.boundary.xInner)*sum(H.boundary.yInner)*sum(H.boundary.zInner),sum(H.boundary.xInner)*sum(H.boundary.yInner)*sum(H.boundary.zInner));
        for j = 1:H.mass.R(2)
            H.mass.Y{j} = H.mass.Y{j}(logical(H.boundary.yInner), logical(H.boundary.yInner));
        end
        for k = 1:H.mass.R(3)
            H.mass.Z{k} = H.mass.Z{k}(logical(H.boundary.zInner), logical(H.boundary.zInner));        
        end
        for i = 1:H.mass.R(1)
            H.mass.X{i} = H.mass.X{i}(logical(H.boundary.xInner), logical(H.boundary.xInner));
        end
        if isfield(opt, 'TT_interpolation') && opt.TT_interpolation == 1
            for i = 1:H.mass.R(1)
                for j = 1:H.mass.R(3)
                    H.massMat = H.massMat + kron(H.mass.Z{j}, kron(H.mass.Y{j+(i-1)*H.mass.R(3)}, H.mass.X{i}));
                end
            end                    
        elseif strcmp(opt.lowRankMethod, 'CPD')
            for i = 1:H.mass.R(1)
                H.massMat = H.massMat + kron(H.mass.Z{i},kron(H.mass.Y{i},H.mass.X{i}));
            end
        else
            for i = 1:H.mass.R(1)
                for j = 1:H.mass.R(2)
                    for k = 1:H.mass.R(3)
                        H.massMat = H.massMat + kron(H.mass.Z{k}, kron(H.mass.Y{j}, H.mass.X{i}))*H.mass.SVDWeights(i,j,k);
                    end
                end
            end
        end
        end
        if opt.stiffness == 1
            H.stiffnessMat =sparse(sum(H.boundary.xInner)*sum(H.boundary.yInner)*sum(H.boundary.zInner),sum(H.boundary.xInner)*sum(H.boundary.yInner)*sum(H.boundary.zInner));
            if isfield(opt, 'TT_interpolation') && opt.TT_interpolation == 1
                for n = 1:9
                    for i = 1:H.stiffness.R(H.stiffness.order(n),1)
                        H.stiffness.X{n}{i} = H.stiffness.X{n}{i}(logical(H.boundary.xInner), logical(H.boundary.xInner));
                    end
                    for j = 1:H.stiffness.R(H.stiffness.order(n),2)
                        H.stiffness.Y{n}{j} = H.stiffness.Y{n}{j}(logical(H.boundary.yInner), logical(H.boundary.yInner));
                    end
                    for k = 1:H.stiffness.R(H.stiffness.order(n),3)
                        H.stiffness.Z{n}{k} = H.stiffness.Z{n}{k}(logical(H.boundary.zInner), logical(H.boundary.zInner));
                    end
                    for i = 1:H.stiffness.R(H.stiffness.order(n),1)
                        for j = 1:H.stiffness.R(H.stiffness.order(n),3)
                            H.stiffnessMat = H.stiffnessMat + kron(H.stiffness.Z{n}{j}, kron(H.stiffness.Y{n}{j+(i-1)*H.stiffness.R(H.stiffness.order(n),3)}, H.stiffness.X{n}{i}));
                        end
                    end
                end
            elseif strcmp(opt.lowRankMethod, 'CPD')
                for n = 1:9
                    for i = 1:H.stiffness.R(H.stiffness.order(n),1)
                        H.stiffness.X{n}{i} = H.stiffness.X{n}{i}(logical(H.boundary.xInner), logical(H.boundary.xInner));
                        H.stiffness.Y{n}{i} = H.stiffness.Y{n}{i}(logical(H.boundary.yInner), logical(H.boundary.yInner));
                        H.stiffness.Z{n}{i} = H.stiffness.Z{n}{i}(logical(H.boundary.zInner), logical(H.boundary.zInner));
                        H.stiffnessMat = H.stiffnessMat + kron(H.stiffness.Z{n}{i},kron(H.stiffness.Y{n}{i},H.stiffness.X{n}{i}));
                    end
                end
            else
                for n = 1:9
                    for j = 1:H.stiffness.R(H.stiffness.order(n),2)
                         H.stiffness.Y{n}{j} = H.stiffness.Y{n}{j}(logical(H.boundary.yInner), logical(H.boundary.yInner));
                    end
                    for k = 1:H.stiffness.R(H.stiffness.order(n),3)
                         H.stiffness.Z{n}{k} = H.stiffness.Z{n}{k}(logical(H.boundary.zInner), logical(H.boundary.zInner));
                    end
                    for i = 1:H.stiffness.R(H.stiffness.order(n),1)
                         H.stiffness.X{n}{i} = H.stiffness.X{n}{i}(logical(H.boundary.xInner), logical(H.boundary.xInner));
                        for j = 1:H.stiffness.R(H.stiffness.order(n),2)
                            for k = 1:H.stiffness.R(H.stiffness.order(n),3)
                                H.stiffnessMat = H.stiffnessMat + kron(H.stiffness.Z{n}{k}, kron(H.stiffness.Y{n}{j}, H.stiffness.X{n}{i}))*H.stiffness.SVDWeights{H.stiffness.order(n)}(i,j,k);
                            end
                        end
                    end
                end
            end
        end
    else
        if opt.stiffness == 1
            for n = 1:9
                for i = 1:H.stiffness.R(H.stiffness.order(n),1)
                    H.stiffness.X{n}{i} = H.stiffness.X{n}{i}(logical(H.boundary.xInner), logical(H.boundary.xInner));
                end
                for j = 1:H.stiffness.R(H.stiffness.order(n),2)
                    H.stiffness.Y{n}{j} = H.stiffness.Y{n}{j}(logical(H.boundary.yInner), logical(H.boundary.yInner));
                end
                for k = 1:H.stiffness.R(H.stiffness.order(n),3)
                    H.stiffness.Z{n}{k} = H.stiffness.Z{n}{k}(logical(H.boundary.zInner), logical(H.boundary.zInner));
                end
            end
        end
        if opt.mass == 1
            for i = 1:H.mass.R(1)
                H.mass.X{i} = H.mass.X{i}(logical(H.boundary.xInner),logical(H.boundary.xInner));
            end
            for j = 1:H.mass.R(2)
                H.mass.Y{j} = H.mass.Y{j}(logical(H.boundary.yInner),logical(H.boundary.yInner));
            end
            for k = 1:H.mass.R(3)
                H.mass.Z{k} = H.mass.Z{k}(logical(H.boundary.zInner),logical(H.boundary.zInner));
            end
        end
    end

end