function geometry = refinement(geometry,n,degree)
%% refinement by n knot insertions, each doubling the number of intervals
% using geoPDEs toolbox for refinement method

if geometry.dim == 3
    nurbs = struct;
    nurbs.form = 'B-NURBS';
    nurbs.knots = geometry.knots;
    nurbs.order = geometry.degree+1;
    nurbs.number = geometry.n;
    nurbs.dim = geometry.dim+1;
    nurbs.coefs = zeros([4,nurbs.number]);
    for i = 1:3
        nurbs.coefs(i,:,:,:) = squeeze(geometry.controlPoints(:,:,:,i).*geometry.weight);
    end
    nurbs.coefs(4,:,:,:) = geometry.weight;
    if nargin > 2
        degelev  = max (degree - (nurbs.order-1), 0);
        nurbs    = nrbdegelev (nurbs, degelev);
    end
    nsub = ones(1,3)*n;
    [~,~, iknots] = kntrefine (nurbs.knots, nsub, nurbs.order-1, nurbs.order-2);
    inurbs = nrbkntins(nurbs,iknots);
    if geometry.nurbs == 1
        geometry = readGeo(inurbs);
        geometry.nurbs = 1;
    else
        geometry = readGeo(inurbs);
    end
else
    nurbs = struct;
    nurbs.form = 'B-NURBS';
    nurbs.knots = geometry.knots;
    nurbs.order = geometry.degree+1;
    nurbs.number = geometry.n;
    nurbs.dim = geometry.dim+1;
    nurbs.coefs = zeros([4,nurbs.number]);
    for i = 1:2
        nurbs.coefs(i,:,:) = squeeze(geometry.controlPoints(:,:,i).*geometry.weight);
    end
    nurbs.coefs(4,:,:) = geometry.weight;
    if nargin > 2
        degelev  = max (degree - (nurbs.order-1), 0);
        nurbs    = nrbdegelev (nurbs, degelev);
    end
    nsub = ones(1,2)*n;
    [~,~,iknots] = kntrefine(nurbs.knots, nsub, nurbs.order-1, nurbs.order-2);
    inurbs = nrbkntins(nurbs,iknots);
    if geometry.nurbs == 1
        geometry = readGeo(inurbs);
        geometry.nurbs = 1;
    else
        geometry = readGeo(inurbs);
    end
end


