function [H, opt] = univariateIntegralXGauss(H, opt)

if nargin < 2
    opt = struct();
end

n = numel(H.geometry.knots{1}) - H.geometry.degree(1) -1;
% Gauss quadrature with 5 points on each nonempty knot span
s = [-0.906179845938664, -0.538469310105683, 0, 0.538469310105683, 0.906179845938664];
w = [0.236926885056189, 0.478628670499366, 0.568888888888889, 0.478628670499366, 0.236926885056189]';
if opt.stiffness == 1
    H.stiffness.X = cell(9,1);
    for i = 1:9
        H.stiffness.X{i} = cell(H.stiffness.R(H.stiffness.order(i),1),1);
        H.stiffness.X{i}(:) = {sparse(n,n)};
    end
end
if opt.mass == 1
    H.mass.X = cell(H.mass.R(1),1);
    H.mass.X(:) = {sparse(n,n)};
end

for l = 1:length(H.geometry.tensor.knots{1})-1
    if H.geometry.tensor.knots{1}(l) < H.geometry.tensor.knots{1}(l+1)
        a = H.geometry.tensor.knots{1}(l);
        b = H.geometry.tensor.knots{1}(l+1);
        xx = (b-a)/2*s + (a+b)/2;
        quadValues = evalNURBS(H.geometry.tensor.knots{1}, H.geometry.tensor.degree(1), H.geometry.tensor.Tweights{1}', xx);
        quadValues2 = evalNURBS(H.weightFun.knots{1}, H.weightFun.degree(1), H.weightFun.weight{1}', xx);
        quadValuesDeriv = evalNURBSDeriv(H.geometry.tensor.knots{1}, H.geometry.tensor.degree(1), H.geometry.tensor.Tweights{1}', xx);
        for i = l-H.geometry.tensor.degree(1):l
            for j = l-H.geometry.tensor.degree(1):l
                for k = 1:length(s)
                    if opt.stiffness == 1 
                        for r = 1:H.stiffness.R(1,1)
                            H.stiffness.X{1}{r}(i,j) = H.stiffness.X{1}{r}(i,j) + ((b-a)/2)*w(k)*quadValuesDeriv(i,k)*quadValuesDeriv(j,k)*quadValues2(:,k)'*H.stiffness.SVDU{1}{1}(:,r);
                        end
                        for r = 1:H.stiffness.R(2,1)
                            H.stiffness.X{2}{r}(i,j) = H.stiffness.X{2}{r}(i,j) + ((b-a)/2)*w(k)*quadValues(i,k)*quadValuesDeriv(j,k)*quadValues2(:,k)'*H.stiffness.SVDU{2}{1}(:,r);
                            H.stiffness.X{4}{r}(i,j) = H.stiffness.X{4}{r}(i,j) + ((b-a)/2)*w(k)*quadValuesDeriv(i,k)*quadValues(j,k)*quadValues2(:,k)'*H.stiffness.SVDU{2}{1}(:,r);
                        end
                        for r = 1:H.stiffness.R(3,1)
                            H.stiffness.X{3}{r}(i,j) = H.stiffness.X{3}{r}(i,j) + ((b-a)/2)*w(k)*quadValues(i,k)*quadValuesDeriv(j,k)*quadValues2(:,k)'*H.stiffness.SVDU{3}{1}(:,r);
                            H.stiffness.X{7}{r}(i,j) = H.stiffness.X{7}{r}(i,j) + ((b-a)/2)*w(k)*quadValuesDeriv(i,k)*quadValues(j,k)*quadValues2(:,k)'*H.stiffness.SVDU{3}{1}(:,r);
                        end
                        for r = 1:H.stiffness.R(4,1)
                            H.stiffness.X{5}{r}(i,j) = H.stiffness.X{5}{r}(i,j) + ((b-a)/2)*w(k)*quadValues(i,k)*quadValues(j,k)*quadValues2(:,k)'*H.stiffness.SVDU{4}{1}(:,r);
                        end
                        for r = 1:H.stiffness.R(5,1)
                            H.stiffness.X{6}{r}(i,j) = H.stiffness.X{6}{r}(i,j) + ((b-a)/2)*w(k)*quadValues(i,k)*quadValues(j,k)*quadValues2(:,k)'*H.stiffness.SVDU{5}{1}(:,r);
                            H.stiffness.X{8}{r}(i,j) = H.stiffness.X{8}{r}(i,j) + ((b-a)/2)*w(k)*quadValues(i,k)*quadValues(j,k)*quadValues2(:,k)'*H.stiffness.SVDU{5}{1}(:,r);
                        end
                        for r = 1:H.stiffness.R(6,1)
                            H.stiffness.X{9}{r}(i,j) = H.stiffness.X{9}{r}(i,j) + ((b-a)/2)*w(k)*quadValues(i,k)*quadValues(j,k)*quadValues2(:,k)'*H.stiffness.SVDU{6}{1}(:,r);
                        end
                    end
                    if opt.mass ==1
                        for r = 1:H.mass.R(1)
                            H.mass.X{r}(i,j) = H.mass.X{r}(i,j) + ((b-a)/2)*w(k)*quadValues(i,k)*quadValues(j,k)*quadValues2(:,k)'*H.mass.SVDU{1}(:,r);
                        end
                    end
                end
            end
        end
    end
end






end