function [H, opt] = univariateIntegralYGauss(H, opt)

if nargin < 2
    opt = struct();
end
if ~isfield(opt, 'quadSize') || isempty(opt.quadSize)
    opt.quadSize = 100;
end
n = numel(H.geometry.knots{2}) - H.geometry.degree(2) -1;
% Gauss quadrature with 5 points on each nonempty knot span
s = [-0.906179845938664, -0.538469310105683, 0, 0.538469310105683, 0.906179845938664];
w = [0.236926885056189, 0.478628670499366, 0.568888888888889, 0.478628670499366, 0.236926885056189]';
        
if opt.stiffness == 1
    H.stiffness.Y = cell(9,1);
    for i = 1:9
        H.stiffness.Y{i} = cell(H.stiffness.R(H.stiffness.order(i),2),1);
        H.stiffness.Y{i}(:) = {zeros(n,n)};
    end
end
if opt.mass == 1
    H.mass.Y = cell(H.mass.R(2),1);
    H.mass.Y(:) = {sparse(n,n)};
end

for l = 1:length(H.geometry.tensor.knots{2})-1
    if H.geometry.tensor.knots{2}(l) < H.geometry.tensor.knots{2}(l+1)
        a = H.geometry.tensor.knots{2}(l);
        b = H.geometry.tensor.knots{2}(l+1);
        xx = (b-a)/2*s + (a+b)/2;
        quadValues = evalNURBS(H.geometry.tensor.knots{2}, H.geometry.tensor.degree(2), H.geometry.tensor.Tweights{2}', xx);
        quadValues2 = evalNURBS(H.weightFun.knots{2}, H.weightFun.degree(2), H.weightFun.weight{2}', xx);
        quadValuesDeriv = evalNURBSDeriv(H.geometry.tensor.knots{2}, H.geometry.tensor.degree(2), H.geometry.tensor.Tweights{2}', xx);
        for i = l-H.geometry.tensor.degree(2):l
            for j = l-H.geometry.tensor.degree(2):l
                for k = 1:length(s)
                    if opt.stiffness == 1 
                        for r = 1:H.stiffness.R(1,2)
                            H.stiffness.Y{1}{r}(i,j) = H.stiffness.Y{1}{r}(i,j) + ((b-a)/2)*w(k)*quadValues(i,k)*quadValues(j,k)*quadValues2(:,k)'*H.stiffness.SVDU{1}{2}(:,r);
                        end
                        for r = 1:H.stiffness.R(2,2)
                            H.stiffness.Y{2}{r}(i,j) = H.stiffness.Y{2}{r}(i,j) + ((b-a)/2)*w(k)*quadValuesDeriv(i,k)*quadValues(j,k)*quadValues2(:,k)'*H.stiffness.SVDU{2}{2}(:,r);
                            H.stiffness.Y{4}{r}(i,j) = H.stiffness.Y{4}{r}(i,j) + ((b-a)/2)*w(k)*quadValues(i,k)*quadValuesDeriv(j,k)*quadValues2(:,k)'*H.stiffness.SVDU{2}{2}(:,r);
                        end
                        for r = 1:H.stiffness.R(3,2)
                            H.stiffness.Y{3}{r}(i,j) = H.stiffness.Y{3}{r}(i,j) + ((b-a)/2)*w(k)*quadValues(i,k)*quadValues(j,k)*quadValues2(:,k)'*H.stiffness.SVDU{3}{2}(:,r);
                            H.stiffness.Y{7}{r}(i,j) = H.stiffness.Y{7}{r}(i,j) + ((b-a)/2)*w(k)*quadValues(i,k)*quadValues(j,k)*quadValues2(:,k)'*H.stiffness.SVDU{3}{2}(:,r);
                        end
                        for r = 1:H.stiffness.R(4,2)
                            H.stiffness.Y{5}{r}(i,j) = H.stiffness.Y{5}{r}(i,j) + ((b-a)/2)*w(k)*quadValuesDeriv(i,k)*quadValuesDeriv(j,k)*quadValues2(:,k)'*H.stiffness.SVDU{4}{2}(:,r);
                        end
                        for r = 1:H.stiffness.R(5,2)
                            H.stiffness.Y{6}{r}(i,j) = H.stiffness.Y{6}{r}(i,j) + ((b-a)/2)*w(k)*quadValues(i,k)*quadValuesDeriv(j,k)*quadValues2(:,k)'*H.stiffness.SVDU{5}{2}(:,r);
                            H.stiffness.Y{8}{r}(i,j) = H.stiffness.Y{8}{r}(i,j) + ((b-a)/2)*w(k)*quadValuesDeriv(i,k)*quadValues(j,k)*quadValues2(:,k)'*H.stiffness.SVDU{5}{2}(:,r);
                        end
                        for r = 1:H.stiffness.R(6,2)
                            H.stiffness.Y{9}{r}(i,j) = H.stiffness.Y{9}{r}(i,j) + ((b-a)/2)*w(k)*quadValues(i,k)*quadValues(j,k)*quadValues2(:,k)'*H.stiffness.SVDU{6}{2}(:,r);
                        end
                    end
                    if opt.mass ==1
                        for r = 1:H.mass.R(2)
                            H.mass.Y{r}(i,j) = H.mass.Y{r}(i,j) + ((b-a)/2)*w(k)*quadValues(i,k)*quadValues(j,k)*quadValues2(:,k)'*H.mass.SVDU{2}(:,r);
                        end
                    end
                end
            end
        end
    end
end


end



