function [H, opt] = univariateIntegralZGauss(H, opt)

if nargin < 2
    opt = struct();
end
if ~isfield(opt, 'quadSize') || isempty(opt.quadSize)
    opt.quadSize = 100;
end
n = numel(H.geometry.knots{3}) - H.geometry.degree(3) -1;
% Gauss quadrature with 5 points on each nonempty knot span
s = [-0.906179845938664, -0.538469310105683, 0, 0.538469310105683, 0.906179845938664];
w = [0.236926885056189, 0.478628670499366, 0.568888888888889, 0.478628670499366, 0.236926885056189]';
        
if opt.stiffness == 1
    H.stiffness.Z = cell(9,1);
    for i = 1:9
        H.stiffness.Z{i} = cell(H.stiffness.R(H.stiffness.order(i),3),1);
        H.stiffness.Z{i}(:) = {zeros(n,n)};
    end
end
if opt.mass == 1
    H.mass.Z = cell(H.mass.R(3),1);
    H.mass.Z(:) = {sparse(n,n)};
end

for l = 1:length(H.geometry.tensor.knots{3})-1
    if H.geometry.tensor.knots{3}(l) < H.geometry.tensor.knots{3}(l+1)
        a = H.geometry.tensor.knots{3}(l);
        b = H.geometry.tensor.knots{3}(l+1);
        xx = (b-a)/2*s + (a+b)/2;
        quadValues = evalNURBS(H.geometry.tensor.knots{3}, H.geometry.tensor.degree(3), H.geometry.tensor.Tweights{3}', xx);
        quadValues2 = evalNURBS(H.weightFun.knots{3}, H.weightFun.degree(3), H.weightFun.weight{3}', xx);
        quadValuesDeriv = evalNURBSDeriv(H.geometry.tensor.knots{3}, H.geometry.tensor.degree(3), H.geometry.tensor.Tweights{3}', xx);
        for i = l-H.geometry.tensor.degree(3):l
            for j = l-H.geometry.tensor.degree(3):l
                for k = 1:length(s)
                    if opt.stiffness == 1 
                        for r = 1:H.stiffness.R(1,3)
                            H.stiffness.Z{1}{r}(i,j) = H.stiffness.Z{1}{r}(i,j) + ((b-a)/2)*w(k)*quadValues(i,k)*quadValues(j,k)*quadValues2(:,k)'*H.stiffness.SVDU{1}{3}(:,r);
                        end
                        for r = 1:H.stiffness.R(2,3)
                            H.stiffness.Z{2}{r}(i,j) = H.stiffness.Z{2}{r}(i,j) + ((b-a)/2)*w(k)*quadValues(i,k)*quadValues(j,k)*quadValues2(:,k)'*H.stiffness.SVDU{2}{3}(:,r);
                            H.stiffness.Z{4}{r}(i,j) = H.stiffness.Z{4}{r}(i,j) + ((b-a)/2)*w(k)*quadValues(i,k)*quadValues(j,k)*quadValues2(:,k)'*H.stiffness.SVDU{2}{3}(:,r);
                        end
                        for r = 1:H.stiffness.R(3,3)
                            H.stiffness.Z{3}{r}(i,j) = H.stiffness.Z{3}{r}(i,j) + ((b-a)/2)*w(k)*quadValuesDeriv(i,k)*quadValues(j,k)*quadValues2(:,k)'*H.stiffness.SVDU{3}{3}(:,r);
                            H.stiffness.Z{7}{r}(i,j) = H.stiffness.Z{7}{r}(i,j) + ((b-a)/2)*w(k)*quadValues(i,k)*quadValuesDeriv(j,k)*quadValues2(:,k)'*H.stiffness.SVDU{3}{3}(:,r);
                        end
                        for r = 1:H.stiffness.R(4,3)
                            H.stiffness.Z{5}{r}(i,j) = H.stiffness.Z{5}{r}(i,j) + ((b-a)/2)*w(k)*quadValues(i,k)*quadValues(j,k)*quadValues2(:,k)'*H.stiffness.SVDU{4}{3}(:,r);
                        end
                        for r = 1:H.stiffness.R(5,3)
                            H.stiffness.Z{6}{r}(i,j) = H.stiffness.Z{6}{r}(i,j) + ((b-a)/2)*w(k)*quadValuesDeriv(i,k)*quadValues(j,k)*quadValues2(:,k)'*H.stiffness.SVDU{5}{3}(:,r);
                            H.stiffness.Z{8}{r}(i,j) = H.stiffness.Z{8}{r}(i,j) + ((b-a)/2)*w(k)*quadValues(i,k)*quadValuesDeriv(j,k)*quadValues2(:,k)'*H.stiffness.SVDU{5}{3}(:,r);
                        end
                        for r = 1:H.stiffness.R(6,3)
                            H.stiffness.Z{9}{r}(i,j) = H.stiffness.Z{9}{r}(i,j) + ((b-a)/2)*w(k)*quadValuesDeriv(i,k)*quadValuesDeriv(j,k)*quadValues2(:,k)'*H.stiffness.SVDU{6}{3}(:,r);
                        end
                    end
                    if opt.mass ==1
                        for r = 1:H.mass.R(3)
                            H.mass.Z{r}(i,j) = H.mass.Z{r}(i,j) + ((b-a)/2)*w(k)*quadValues(i,k)*quadValues(j,k)*quadValues2(:,k)'*H.mass.SVDU{3}(:,r);
                        end
                    end
                end
            end
        end
    end
    
end


end