% optimal control experiment with flag domain

addpath(genpath('CodesIGA'))

%% set up system matrices
ExampleSystemGeneration;
 
% Parameters for the optimization
param = struct;
param.nt = 10;          % integer, number of time steps
param.T = 1;            % end time of time interval
param.tol = 1e-5;       % error tolerance for optimal control problem
param.beta = 1e-2;      % control regularization parameter
param.preconditioner = 0; % 0 or 1, use preconditioning (preconditioner is not ready yet)
dt = param.T/param.nt;

%% Set up a desired state for the control problem
f = @(x,y,z) exp(( - (x-1).^2 - (y-1).^2 - (z-0.4).^2)*4); % fitting desired state for quarter annulus domain
% f = @(x,y,z) 2*exp(( - (x-2).^2 - (y-2).^2 - (z-4).^2)); % fitting desired state function for 3Dflag domain
% Interpolation of the desired state by B-splines
grevillePoints = cell(H.geometry.dim,1);
grevilleValues = cell(H.geometry.dim,1);
for i=1:H.geometry.dim
    grevillePoints{i} = generateGrevillePoints(H.geometry.knots{i},H.geometry.degree(i));
    grevilleValues{i} =  sparse(evalNURBS      (H.geometry.tensor.knots{i}, H.geometry.tensor.degree(i), H.geometry.tensor.Tweights{i}', grevillePoints{i}));
end
MM = kron(grevilleValues{3},kron(grevilleValues{2},grevilleValues{1}))';
cc= reshape(H.geometry.controlPoints,prod(H.geometry.n),3);
Gx = MM*cc(:,1);
Gy = MM*cc(:,2);
Gz = MM*cc(:,3);
yhat = f(Gx,Gy,Gz);
yhat(logical(~(kron(H.boundary.zInner, kron(H.boundary.yInner, H.boundary.xInner))))) = 0;
yhat = yhat(logical(kron(H.boundary.zInner, kron(H.boundary.yInner, H.boundary.xInner))));

%% optimal control with low rank system
[sol_TT, max_res, swp, max_rank] = optimalControl_TT(H,yhat,param);

% solution sol_TT is in TT-format
solFull = full(sol_TT);
% plot state solution at arbitrary time step
t = ceil(param.nt/2);
plotNSol(H,solFull(1,size(solFull,2)/param.nt*(t-1)+1:size(solFull,2)/param.nt*t), yhat);
