addpath(genpath('CodesIGA'))

%% load the geometry file
% load('3Dflag.mat'); % Geometry file not in geoPDEs format
load('3Dflag_refined.mat'); % same geometry as 3Dflag.mat with 5 h-refinements
% G = readGeo('geo_thick_ring.txt'); % the function readGeo lets you read in geometry files in geoPDEs format if the geoPDEs toolbox is installed.

%% set up system options
% options for the matrix assembly
opt = struct;
opt.rankTol = 1e-5;     % tolerance for rank truncation
opt.refinement = 0;     % integer, number of h-refinements
opt.discardFull = 1;    % 0 or 1, discards additional data after assembly and does not assemble the full matrices, change to 0 to get full system matrices
opt.plotW =  0;         % 0 or 1, plots the interpolation of the weight functions
opt.lowRank = 1;        % 0 or 1, reduces the rank by low rank approximation
opt.mass = 1;           % 0 or 1, computes the weight matrix
opt.stiffness = 1;      % 0 or 1, computes the stiffness matrix
opt.sizeLowRank = [];   % integer, pre-set a fixed rank size
opt.elevation = [2,2,2];% array of 3 integers, degree elevation in each geometric dimension
opt.lowRankMethod = 'TT';% 'TT' or 'CPD', Method for the low rank assembly
opt.quadSize = 100;     % only necessary if plot of weight functions is selected
opt.greville = 1;       % exact interpolation on the greville Points, can be shifted by setting to different value 
opt.TT_interpolation = 1;% Do the interpolation step in a fast way by using TT-approximation


%% Generation of the system matrices
[H,opt] = system_generation(G,opt); % H now holds all information on the system including the system matrices

% plot the geometry:
p = figure;
title('3D Geometry')
p = plotNDomain3(G,p);
p = plotNLines3(G,'black',p);
